;**********************************************************************
;   SV012.asm - servo tester						
;
;   Servo connected to PA2 controlled by buttons on RB4 and RB5
;   Servo connected to PA3 controlled by buttons on RB6 and RB7
;
;	Servos need a 1.5ms pulse every 20ms
;
;	Vary the width of the 1.5ms pulse to change the position.
;
;	If the pulses are too close together, then the servo can get confused
;	If the pulses are too far apart, the servo may be rough.
;	
;
;   Port A2 provides a pulse from 1ms to 2ms controlled by PB4 and PB5
;   Port A3 provides a pulse from 1ms to 2ms controlled by PB6 and PB7
;
;   These pulses shoule be repeated about every 20ms
;
;   If you make PB6 or PB7 low PA3's pulse width changes:
;	PB6 Clockwise
;	PB7 Anticlockwise
;	PB6 and PB7 center
;					
;   If you make PB4 or PB5 low PA2's pulse width changes:
;	PB4 Clockwise
;	PB5 Anticlockwise
;	PB4 and PB5 center

;
;**********************************************************************
;
;    Filename:	  SV012.asm
;    Date:          29th September 2005
;    File Version:  Draft 1a
;
;    Author:        Doug Rice
;    Company:       DougTronics
;
;
;**********************************************************************
;
;    Files required:                                                  *
;
;
;
;**********************************************************************
;
;    Notes: 
;
;	                       
;
;           
;**********************************************************************


	list      p=16f628            ; list directive to define processor
	#include <p16f628.inc>        ; processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _INTRC_OSC_CLKOUT  & _MCLRE_OFF & _LVP_OFF

;	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _ER_OSC_CLKOUT  & _MCLRE_OFF & _LVP_OFF
	
;	__CONFIG 0x3FFF

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.





;***** VARIABLE DEFINITIONS
w_temp        EQU     0x70        ; variable used for context saving 
status_temp   EQU     0x71        ; variable used for context saving


BUTTON_UP2	EQU 5
BUTTON_DOWN2	EQU 4
BUTTON_BOTH2	EQU 4

BUTTON_UP3	EQU 7
BUTTON_DOWN3	EQU 6
BUTTON_BOTH3	EQU 6


	cblock	0x20	; 16F628

	  ; delay variables
	  DLdelcnt
	  DLdelcnt1

	  ; position memories
	  T3pos
	  T3poscnt

	  ; position memories
	  T2pos
	  T2poscnt


	  ; Input variables
	  IPnew
	  IPlast
	  IPbuttonEvent
	  IPbuttonEvent2
	  
;	  PORTBT
	
	endc
	
PORTBT EQU	PORTB

;**********************************************************************
		ORG     0x000             ; processor reset vector
		goto    main              ; go to beginning of program


		ORG     0x004             ; interrupt vector location
		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register

; isr code can go here or be located as a call subroutine elsewhere

		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt



main
	; Servo Loop. 
	; PORT B bits have one of 8 settings
	; The pulse widths on PORT B vary from 1ms to 2ms in 125us steps.
	; this seems to work.

	CLRF PORTA ;Initialize PORTA by setting
	;output data latches
	BANKSEL CMCON
	MOVLW 0X07 ;Turn comparators off and
	MOVWF CMCON ;enable pins for I/O


	BANKSEL OPTION_REG
	BCF	OPTION_REG,NOT_RBPU

	BANKSEL 	TRISB
	CLRF	TRISB	
	MOVLW	0x00
	MOVWF	TRISA

	MOVLW	0xF0
	MOVWF	TRISB

     BANKSEL PORTB
	CLRF	PORTA
	CLRF	PORTB


	; set up initial position to mid point
	MOVLW	0x07
	MOVWF	T3pos
	MOVWF	T2pos


mainLoop

	CALL T2	; Apply Pulse to Servo on PA2
	CALL T3	; Apply Pulse to Servo on PA3

	
	; http://www.rc-cam.com/servotst.htm says that the frame rate should be about 20ms
	; If it is too short the servo may be jerky.
	; padd out the fram a bit
	CALL	DLdelay2ms
	CALL	DLdelay2ms
	CALL	DLdelay2ms
	CALL	DLdelay2ms

	call	IPtimeslice

	CALL	DLdelay2ms

	

	; Tests for buttons that control Servo on PA2
	BTFSS	IPbuttonEvent,BUTTON_UP2	; On BUTTON_SET Press
	CALL	IPupPressed2

	BTFSS	IPbuttonEvent,BUTTON_DOWN2	; On BUTTON_NEXT Press
	CALL	IPdownPressed2

	BTFSS	IPbuttonEvent2,BUTTON_BOTH2	; On BUTTON_BOTH Press
	CALL	IPbothPressed2



	; Tests for buttons that control Servo on PA2
	BTFSS	IPbuttonEvent,BUTTON_UP3	; On BUTTON_SET Press
	CALL	IPupPressed3

	BTFSS	IPbuttonEvent,BUTTON_DOWN3	; On BUTTON_NEXT Press
	CALL	IPdownPressed3

	BTFSS	IPbuttonEvent2,BUTTON_BOTH3	; On BUTTON_BOTH Press
	CALL	IPbothPressed3

	GOTO	mainLoop





T2
	;
	; Apply a pulse width of 500us+ ( T3pos*125us ) on PA2
	; connect Servo control to a pin to set position.
	; 
	; 

	MOVFW	T2pos
	MOVWF	T2poscnt
	INCF	T2poscnt,F
	BSF		PORTA,2
	call 	DLdelay500us
T2loop10
	DECFSZ	T2poscnt,f
	GOTO	T2loop11
	goto	T2loop12
T2loop11
	call 	DLdelay125us
	goto	T2loop10
T2loop12
	BCF		PORTA,2

	CALL	DLdelay2ms
	CALL	DLdelay2ms

	return



T3
	;
	; Apply a pulse width of 500us+ ( T3pos*125us ) on PA3
	; connect Servo control to a pin to set position.
	; 
	; 

	MOVFW	T3pos
	MOVWF	T3poscnt
	INCF	T3poscnt,F
	BSF	PORTA,3
	call DLdelay500us
T3loop10
	DECFSZ	T3poscnt,f
	GOTO	T3loop11
	goto	T3loop12
T3loop11
	call 	DLdelay125us
	goto	T3loop10
T3loop12
	BCF		PORTA,3
	
	CALL	DLdelay2ms
	CALL	DLdelay2ms
	
	return


; For Servo PA2

IPupPressed2
	BSF	IPbuttonEvent,BUTTON_UP2
	INCF	T2pos,w
	ANDLW	0x0F
	; limit and prevent wrap around
	SKPNZ
	MOVLW	0x0F
	MOVWF	T2pos	

	return

IPdownPressed2
	BSF	IPbuttonEvent,BUTTON_DOWN2
	DECF	T2pos,w
	ANDLW	0x0F
	; limit and prevent wrap around
	SKPNZ
	MOVLW	0x01
	MOVWF	T2pos	
	return	

IPbothPressed2
	; center the servo
	BSF	IPbuttonEvent2,BUTTON_BOTH2
	MOVLW	0x07
	MOVWF	T2pos	
	return	


IPupPressed3
	BSF	IPbuttonEvent,BUTTON_UP3
	INCF	T3pos,w
	ANDLW	0x0F
	; limit and prevent wrap around
	SKPNZ
	MOVLW	0x0F
	MOVWF	T3pos	

	return


; For Servo PA3

IPdownPressed3
	BSF	IPbuttonEvent,BUTTON_DOWN3
	DECF	T3pos,w
	ANDLW	0x0F
	; limit and prevent wrap around
	SKPNZ
	MOVLW	0x01
	MOVWF	T3pos	
	return	

IPbothPressed3
	; center the servo
	BSF	IPbuttonEvent2,BUTTON_BOTH3
	MOVLW	0x07
	MOVWF	T3pos	
	return	



DLdelay100us
	movlw	0x0D
	goto 	DLdelayShort

DLdelay125us
	movlw	0x11
	goto 	DLdelayShort

DLdelay500us
	movlw	0x46
	goto 	DLdelayShort

DLdelay1ms
	movlw	0x80
;;	movlw	0x90 ; this is 1ms
	goto 	DLdelayShort

DLdelay2ms
	movlw	0xFF ; longest
	goto 	DLdelayShort


DLdelayShort
	movwf	DLdelcnt
DLdelayShort1
	GOTO $+1
	GOTO $+1
	DECFSZ	DLdelcnt,f
	goto	DLdelayShort1
	return


IPtimeslice
		;	----___
		;	--_____
		;
		;	Single button event of falling edge, ie when button pressed
                ;       both button events when both buttons are found to be low		

		; This reads all Port B inputs and looks for Press
		MOVFW	IPnew
		MOVWF	IPlast
		MOVFW	PORTBT
		MOVWF	IPnew
		; IP last contains new setting, IPlast contains previous
		COMF	IPlast,W
		IORWF	IPnew,W
		; now force IPbuttonEvent bits to low for new pressed button
		; the service routine should set the bit to clear the event.
		ANDWF	IPbuttonEvent,F

		
		; LOOK FOR TWO BUTTONS PRESSED.
		; if both buttons are low on both scans then bit will be low 		
		MOVFW	IPnew
		IORWF	IPlast,w

		; now mask  off bits of interest
		ANDLW	1<<BUTTON_UP2 | 1 << BUTTON_DOWN2

		SKPZ
		goto 	IPtimeslice1
		
		BCF	IPbuttonEvent2,BUTTON_BOTH2
		; if both buttons low it's a BUTTON_BOTH2 event, 
		; so cancel any BUTTON_UP2 and BUTTON_DOWN2 events.
		MOVLW	1<<BUTTON_UP2 | 1 << BUTTON_DOWN2
		IORWF	IPbuttonEvent,F

		goto 	IPtimeslice1

IPtimeslice1		
		; LOOK FOR TWO BUTTONS PRESSED.
		; if both buttons are low on both scans then bit will be low 		
		MOVFW	IPnew
		IORWF	IPlast,w
		; now mask  off bits of interest
		ANDLW	1<<BUTTON_UP3 | 1 << BUTTON_DOWN3

		SKPZ
		RETURN	
		
		BCF	IPbuttonEvent2,BUTTON_BOTH3
		; if both buttons low it's a BUTTON_BOTH3 event, 
		; so cancel any BUTTON_UP3 and BUTTON_DOWN3 events.
		MOVLW	1<<BUTTON_UP3 | 1 << BUTTON_DOWN3
		IORWF	IPbuttonEvent,F

		RETURN
		

		END                       ; directive 'end of program'

